{***************************************************************************}
{                      UNIDAD DE SVGA BIOS EXTENSION

                               Copyright (c) 1997,98 Marco Antonio Alvarado

Utiliza VESA Super VGA BIOS Extension 1.2.}
{***************************************************************************}

{Revision. Permite revisar el resultado de las funciones. Maneja los errores
haciendo lentas las funciones y terminando el programa si funciona mal.}
{$define revision}

unit usvbe;

interface

  const
    soporte = $4f;
    exito = $00;
    todosestados = $0f;
    estadoequipo = $01;
    estadobios = $02;
    estadodac = $04;
    estadosvga = $08;
    unidadestado = 64;
    unidadmemoria = 65536;
    unidadgranularidad = 1024;

  type
{    pchar = ^char;  {Turbo Pascal 7.0 lo define.}
    pword = ^word;

{Rinforme. Contiene la informacin general sobre SVGA. Asgnelo con informa.}
    rinforme = record

{Identidad. Contiene 'VESA' si el informe es vlido.}
      identidad : array [0..3] of char;

{Versionmenor y versionmayor. Nmeros de la versin.}
      versionmenor : byte;
      versionmayor : byte;

{Frase. Apunta a la frase del creador terminada con 0. Puede usarse para
identificar la tarjeta y sus caractersticas.}
      frase : pchar;

{Capacidad. Describe las caractersticas que soporta el ambiente de video.
Contenido:

  Binario 0 = tamao de las intensidades.
  Binarios 1 a 31 = reservados.

  Tamao de las intensidades (binario 0):
    0 = el tamao de las intensidades es 6 binarios.
    1 = el tamao de las intensidades puede alterarse, as que puede usar
      ponetamanointensidad.}
      capacidad : array [0..3] of byte;

{Modo. Apunta a una lista de modos soportados, termina con -1 ($ffff).
Algunos de los modos pueden no estar soportados debido a la configuracin
actual de la tarjeta de video o el monitor.}
      modo : pword;

{Memoria. Cantidad total de memoria instalada en la tarjeta VGA en unidades
de 65536 letras.}
      memoria : word;

{Resto. Campo reservado para completar el registro a 256 letras.}
      resto : array [0..235] of byte;
    end;

{Rinformemodo. Contiene la informacin sobre un modo de SVGA. Asgnelo con
informamodo.}
    rinformemodo = record

{Atributo. Caractersticas del modo de video. Contenido:

  Binario 0 = soporte de modo.
  Binario 1 = reservado.
  Binario 2 = funciones de salida soportadas.
  Binario 3 = modo monocromo o policromo.
  Binario 4 = modo texto o grfico.
  Binarios 5 a 15 = reservados.

  Soporte de modo (binario 0):
    0 = no es soportado por el equipo.
    1 = es soportado.

  Funciones de salida soportadas (binario 2):
    0 = no son soportadas por BIOS.
    1 = son soportadas.

  Modo monocromo o policromo (binario 3).
    0 = monocromo.
    1 = policromo.

  Modo texto o grfico (binario 4).
    0 = texto.
    1 = grfico.}
      atributo : word;

{Atributo1 y atributo2. Caractersticas de las ventanas 1 y 2. Contenido:

  Binario 0 = Soporte
  Binario 1 = Lectura
  Binario 2 = Escritura

  Soporte (binario 0):
    0 = la ventana no es soportada. Si las dos ventanas no son soportadas
      puede asumir que la memoria de video est en la direccin
      convencional para modomemoria.
    1 = es soportada.

  Lectura (binario 1):
    0 = la ventana no es legible.
    1 = es legible.

  Escritura (binario 2):
    0 = la ventana no es escribible.
    1 = es escribible.}
      atributo1 : byte;
      atributo2 : byte;

{Granularidad. Lmite en unidades de 1024 letras donde puede colocarse la
ventana en la memoria de video. Este valor es indefinido si soporte (binario
0) de atributo1 es 0.}
      granularidad : word;

{Tamano. Tamao de la ventana en unidades de 1024 letras.}
      tamano : word;

{Segmento1 y segmento2. Direcciones de memoria donde estn las ventanas.}
      segmento1 : word;
      segmento2 : word;

{Funciondirecta. Direccin de la funcin que maneja la ventana.}
      funciondirecta : pointer;

{Tamanolinea. Tamao de la lnea lgica en letras, puede ser igual o mayor
que la lnea desplegable.}
      tamanolinea : word;

{Ancho y alto. Resolucin de la pantalla en smbolos (modo texto) o puntos
(modo grfico).}
      ancho : word;
      alto : word;

{Anchosimbolo y altosimbolo. Medidas de los smbolos en puntos.}
      anchosimbolo : byte;
      altosimbolo : byte;

{Planos. Nmero de planos de memoria disponibles.}
      planos : byte;

{Tamanopunto. Cantidad de binarios que forman un punto.}
      tamanopunto : byte;

{Bancos. Nmero de bancos en que se agrupan las lneas. Para modos sin bancos
($0d a $13) debe ser 1.}
      bancos : byte;

{Modomemoria. Organizacin de la memoria. Valores:

  $00 = texto.
  $01 = grficos CGA.
  $02 = grficos Hercules.
  $03 = 4 planos.
  $04 = punto compacto.
  $05 = sin encadenar, 256 colores.
  $06 = color directo.
  $07 = YUV.
  $08 a $ff = reservados.}
      modomemoria : byte;

{Tamanobanco. Especifica el tamao del banco (grupos de lneas) en unidades
de 1024 letras. Para modos sin bancos ($0d a $13) debe ser 0.}
      tamanobanco : byte;

{Imagenes. Cantidad de imagenes adicionales que caben en la memoria de video
al mismo tiempo.}
      imagenes : byte;

{Reserva. Debe ser 1.}
      reserva : byte;

{Tamanorojo, lugarrojo, tamanoverde, lugarverde, tamanoazul, lugarazul y
tamanoreserva. Cantidad de binarios y posicin dentro del punto para las
intensidades de rojo, verde y azul que forman el color direto o YUV.}
      tamanorojo : byte;
      lugarrojo : byte;
      tamanoverde : byte;
      lugarverde : byte;
      tamanoazul : byte;
      lugarazul : byte;
      tamanoreserva : byte;

{Atributocolor. Caractersticas del color directo. Contenido:

  Binario 0 = color directo o tabla
  Binario 1 = utilidad de reserva

  Color directo o tabla (binario 0):
    0 = el color es fijo y se seleciona con color directo
    1 = el color es variable y se almacena en la tabla de la paleta

  Utilidad de reserva (binario 1):
    0 = los binarios de reserva no estn disponibles
    1 = estn disponibles}
      atributocolor : byte;

{Resto. Campo reservado para completar el registro a 256 letras.}
      resto : array [0..216] of byte;
    end;

  var
{Error:
  1 = el video no tiene SVGA BIOS Extension.}
    error : word;

{Resultado. Compare los campos exito y soporte con las constantes del mismo
nombre para determinar si hubo error al ejecutar una rutina. Si el
smbolo condicional revision est definido los errores sern verificados
inmediatamente.}
    resultado : record
      soporte : byte;
      exito : byte;
    end;

    informe : rinforme;

{Funciondirecta. Apunta a la funcin que maneja la ventana. El trabajo es ms
rpido cuando se la llama. Asgnele el campo funciondirecta de rinforme antes
de usar ponelugarventanadirecta y tomalugarventanadirecta.}
    funciondirecta : pointer;

{Informa. Retorna la informacin de SVGA.}
  procedure informa (var hacia : rinforme);

{Informamodo. Retorna la informacin de un modo SVGA. Valores de modo:

                Tipo:     Resolucin:    Colores:
  $0100         Grfico   640 * 400      256
  $0101         Grfico   640 * 480      256   <-- preferido
  $0102 y $6a   Grfico   800 * 600      16
  $0103         Grfico   800 * 600      256
  $0104         Grfico   1024 * 768     16
  $0105         Grfico   1024 * 768     256
  $0106         Grfico   1280 * 1024    16
  $0107         Grfico   1280 * 1024    256
  $0108         Texto     80 * 60        16
  $0109         Texto     132 * 25       16
  $010a         Texto     132 * 43       16
  $010b         Texto     132 * 50       16
  $010c         Texto     132 * 60       16
  $010d         Grfico   320 * 200      32K   (1:5:5:5)
  $010e         Grfico   320 * 200      64K   (5:6:5)
  $010f         Grfico   320 * 200      16.8M (8:8:8)
  $0110         Grfico   640 * 480      32K   (1:5:5:5)
  $0111         Grfico   640 * 480      64K   (5:6:5)
  $0112         Grfico   640 * 480      16.8M (8:8:8)
  $0113         Grfico   800 * 600      32K   (1:5:5:5)
  $0114         Grfico   800 * 600      64K   (5:6:5)
  $0115         Grfico   800 * 600      16.8M (8:8:8)
  $0116         Grfico   1020 * 768     32K   (1:5:5:5)
  $0117         Grfico   1020 * 768     64K   (5:6:5)
  $0118         Grfico   1020 * 768     16.8M (8:8:8)
  $0119         Grfico   1280 * 1024    32K   (1:5:5:5)
  $011a         Grfico   1280 * 1024    64K   (5:6:5)
  $011b         Grfico   1280 * 1024    16.8M (8:8:8)}
  procedure informamodo (modo : word; var hacia : rinformemodo);

{Ponemodo. Pone el modo SVGA. Puede poner a 1 el binario 7 de modo para
prevenir que borre la pantalla.}
  procedure ponemodo (modo : word);

{Tomamodo. Toma el modo SVGA actual. El valor del binario 7 siempre es 0.}
  function tomamodo : word;

{Tomatamanoestados. Regresa el tamao de la estrutura para almacenar los
estados en unidades de 64 letras. Contenido de estado:

  Binario 0 = estado del equipo
  Binario 1 = estado de BIOS
  Binario 2 = estado de DAC
  Binario 3 = estado de SVGA

  Estado del equipo (binario 0):
    0 = no lo usa.
    1 = salva o pone el estado del equipo.

  Estado de BIOS (binario 1):
    0 = no lo usa.
    1 = salva o pone el estado de Bios Input Output System.

  Estado de DAC (binario 2):
    0 = no lo usa.
    1 = salva o pone el estado de Digital to Analog Converter.

  Estado de SVGA (binario 3):
    0 = no lo usa.
    1 = salva o pone el estado de Super Video Graphic Array.}
  function tomatamanoestados (estado : word) : word;

{Salvaestados. Salva los estados de SVGA.}
  procedure salvaestados (estado : word; hacia : pointer);

{Poneestados. Recupera los estados de SVGA.}
  procedure poneestados (estado : word; desde : pointer);

{Ponelugarventana. Posiciona la ventana. Ventana es 0 para la primera ventana
o 1 para la segunda. Lugar es la posicin de la ventana en unidades de
granularidad.}
  procedure ponelugarventana (ventana : byte; lugar : word);

{Tomalugarventana. Retorna la posicin de la ventana en unidades de
granularidad.}
  function tomalugarventana (ventana : byte) : word;

{Ponelugarventanadirecta. Posiciona la ventana llamando directamente a la
funcin que maneja la ventana. Es ms rpido que ponelugarventana. Requiere
funciondirecta.}
  procedure ponelugarventanadirecta (ventana : byte; lugar : word);

{Tomalugarventanadirecta. Retorna la posicin de la ventana llamando
directamente a la funcin que maneja la ventana. Es ms rpido que
tomalugarventana. Requiere funciondirecta.}
  function tomalugarventanadirecta (ventana : byte) : word;

{Ponepuntoslinea. Establece el largo de la lnea lgica en puntos. Permite
establecer una estructura de memoria de video que es ms ancha que el area
desplegada. Poneiniciopantalla pone la posicin inicial desde donde se
despliega. Si no se puede establecer el largo requerido seleccionar el
tamao que se ajuste. Nuevotamano, nuevospuntos y nuevacantidad regresan los
nuevos valores asignados.}
  procedure ponepuntoslinea (puntos : word; var nuevotamano, nuevospuntos,
    nuevacantidad : word);

{Tomapuntoslinea. Retorna el largo de la lnea lgica.}
  procedure tomapuntoslinea (var tamano, puntos, cantidad : word);

{Poneiniciopantalla. Selecciona el punto que se despliega en la esquina
superior izquierda de la pantalla. Puede usarla para desplazar las pantallas
lgicas ms grandes que la visible y cambiar entre pantallas previamente
dibujadas animando.}
  procedure poneiniciopantalla (linea, punto : word);

{Tomainiciopantalla. Retorna el inicio de la pantalla.}
  procedure tomainiciopantalla (var linea, punto : word);

{Ponetamanointensidad. Selecciona el modo de operacin de la paleta DAC.
Esta funcin est disponible si tamao de las intensidades (binario 0) de
capacidad en rinforme es 1. Si no es capaz de cambiar el ancho de la paleta
al valor requerido seleccionar el siguiente valor ms pequeo. Regresa el
nuevo valor asignado.}
  function ponetamanointensidad (tamanointensidad : byte) : byte;

{Tomatamanointensidad. Toma el modo de operacin de la paleta DAC.}
  function tomatamanointensidad : byte;

implementation

  const
    interrupcionvideo = $10;

    funcioninforma = $4f00;
    funcioninformamodo = $4f01;
    funcionponemodo = $4f02;
    funciontomamodo = $4f03;
    funcionmanejaestados = $4f04;
    funciontomatamanoestados = $00;
    funcionsalvaestados = $01;
    funcionponeestados = $02;
    funcionmanejaventana = $4f05;
    funcionponelugarventana = $00;
    funciontomalugarventana = $01;
    funcionmanejapuntoslinea = $4f06;
    funciontomapuntoslinea = $00;
    funcionponepuntoslinea = $01;
    funcionmanejainiciopantalla = $4f07;
    funcionponeiniciopantalla = $00;
    funciontomainiciopantalla = $01;
    funcionmanejatamanointensidad = $4f08;
    funcionponetamanointensidad = $00;
    funciontomatamanointensidad = $01;

{$ifdef revision}
  procedure revisa;
  begin

    if resultado. soporte <> soporte then
    begin
      writeln ('ERROR: Funcin de SVGA BIOS Extension no es soportada.');
      halt;
    end;

    if resultado. exito <> exito then
    begin
      writeln ('ERROR: Funcin de SVGA BIOS Extension fall.');
      halt;
    end;

  end;
{$endif}

  procedure informa (var hacia : rinforme); assembler;
  asm
    mov ax, funcioninforma
    les di, hacia
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  procedure informamodo (modo : word; var hacia : rinformemodo); assembler;
  asm
    mov ax, funcioninformamodo
    mov cx, modo
    les di, hacia
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  procedure ponemodo (modo : word); assembler;
  asm
    mov ax, funcionponemodo
    mov bx, modo
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  function tomamodo : word; assembler;
  asm
    mov ax, funciontomamodo
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov ax, bx
  end;

  function tomatamanoestados (estado : word) : word; assembler;
  asm
    mov ax, funcionmanejaestados
    mov dl, funciontomatamanoestados
    mov cx, estado
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov ax, bx
  end;

  procedure salvaestados (estado : word; hacia : pointer); assembler;
  asm
    mov ax, funcionmanejaestados
    mov dl, funcionsalvaestados
    mov cx, estado
    les bx, hacia
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  procedure poneestados (estado : word; desde : pointer); assembler;
  asm
    mov ax, funcionmanejaestados
    mov dl, funcionponeestados
    mov cx, estado
    les bx, desde
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  procedure ponelugarventana (ventana : byte; lugar : word); assembler;
  asm
    mov ax, funcionmanejaventana
    mov bh, funcionponelugarventana
    mov bl, ventana
    mov dx, lugar
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  function tomalugarventana (ventana : byte) : word; assembler;
  asm
    mov ax, funcionmanejaventana
    mov bh, funciontomalugarventana
    mov bl, ventana
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov ax, dx
  end;

  procedure ponelugarventanadirecta (ventana : byte; lugar : word);
    assembler;
  asm
    mov ax, funcionmanejaventana
    mov bh, funcionponelugarventana
    mov bl, ventana
    mov dx, lugar
    call funciondirecta
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  function tomalugarventanadirecta (ventana : byte) : word; assembler;
  asm
    mov ax, funcionmanejaventana
    mov bh, funciontomalugarventana
    mov bl, ventana
    call funciondirecta
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov ax, dx
  end;

  procedure ponepuntoslinea (puntos : word; var nuevotamano, nuevospuntos,
    nuevacantidad : word); assembler;
  asm
    mov ax, funcionmanejapuntoslinea
    mov bl, funcionponepuntoslinea
    mov cx, puntos
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    les di, nuevotamano
    mov es:word ptr [di], bx
    les di, nuevospuntos
    mov es:word ptr [di], cx
    les di, nuevacantidad
    mov es:word ptr [di], dx
  end;

  procedure tomapuntoslinea (var tamano, puntos, cantidad : word); assembler;
  asm
    mov ax, funcionmanejapuntoslinea
    mov bl, funciontomapuntoslinea
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    les di, tamano
    mov es:word ptr [di], bx
    les di, puntos
    mov es:word ptr [di], cx
    les di, cantidad
    mov es:word ptr [di], dx
  end;

  procedure poneiniciopantalla (linea, punto : word); assembler;
  asm
    mov ax, funcionmanejainiciopantalla
    xor bh, bh  {Debe ser 0.}
    mov bl, funcionponeiniciopantalla
    mov cx, punto
    mov dx, linea
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
  end;

  procedure tomainiciopantalla (var linea, punto : word); assembler;
  asm
    mov ax, funcionmanejainiciopantalla
    mov bl, funciontomainiciopantalla
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
{Bh debe ser 0.}
    les di, punto
    mov es:word ptr [di], cx
    les di, linea
    mov es:word ptr [di], dx
  end;

  function ponetamanointensidad (tamanointensidad : byte) : byte;
    assembler;
  asm
    mov ax, funcionmanejatamanointensidad
    mov bl, funcionponetamanointensidad
    mov bh, tamanointensidad
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov al, bh  {Da el resultado del tamao de las intensidades.}
  end;

  function tomatamanointensidad : byte; assembler;
  asm
    mov ax, funcionmanejatamanointensidad
    mov bl, funciontomatamanointensidad
    int interrupcionvideo
    mov resultado, ax
{$ifdef revision}
    call revisa
{$endif}
    mov al, bh
  end;

begin
  error := 0;
  resultado. exito := exito;
  resultado. soporte := soporte;
  fillchar (informe, sizeof (informe), 0);
  informa (informe);

  if informe. identidad <> 'VESA' then
  begin
    error := 1;
  end;

  funciondirecta := nil;
end.